# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_hyp_wide_R.dat", na.string='.')

# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'comp2','comp3','comp4','comp5','comp6','comp7','comp8',
                         'rec2','rec3','rec4','rec5','rec6','rec7','rec8',
                         'bpi2','bpi3','bpi4','bpi5','bpi6','bpi7','bpi8',
                         'asl2','asl3','asl4','asl5','asl6','asl7','asl8',
                         'ax2','ax3','ax4','ax5','ax6','ax7','ax8',
                         'hds2','hds3','hds4','hds5','hds6','hds7','hds8',
                         'hyp2','hyp3','hyp4','hyp5','hyp6','hyp7','hyp8',
                         'dpn2','dpn3','dpn4','dpn5','dpn6','dpn7','dpn8',
                         'wdn2','wdn3','wdn4','wdn5','wdn6','wdn7','wdn8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

library(OpenMx)

################# Bivariate Linear Growth Model ################# 
blgm.math.hyp.omx <- mxModel("Bivariate Linear Growth Model, Path Specification",
 type="RAM", mxData(observed=nlsy_math_wide, type="raw"),
 manifestVars=c("math2","math3","math4","math5","math6","math7","math8",
                "hyp2","hyp3","hyp4","hyp5","hyp6","hyp7","hyp8"),
 latentVars=c("eta_1","eta_2","eta_3","eta_4"),

# residual variances
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th_M"),

mxPath(from=c("hyp2","hyp3","hyp4","hyp5","hyp6","hyp7","hyp8"),
 arrows=2, free=TRUE, values=1, labels="th_H"),

# latent variable variances and covariances
mxPath(from=c("eta_1","eta_2","eta_3","eta_4"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(50,0,0,0,1,0,0,1,0,1),
 labels=c("psi_11","psi_21","psi_31","psi_41",
                   "psi_22","psi_32","psi_42",
                            "psi_33","psi_43",
                                     "psi_44")),

# factor loadings
mxPath(from="eta_1",
 to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2",
 to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

mxPath(from="eta_3",
 to=c("hyp2","hyp3","hyp4","hyp5","hyp6","hyp7","hyp8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_4",
 to=c("hyp2","hyp3","hyp4","hyp5","hyp6","hyp7","hyp8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts
mxPath(from="one", to=c("eta_1","eta_2","eta_3","eta_4"),
 arrows=1, free=TRUE, values=c(100, 15, 1, 0),
 labels=c("alpha_1","alpha_2","alpha_3","alpha_4")),

# residual covariances
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 to=c("hyp2","hyp3","hyp4","hyp5","hyp6","hyp7","hyp8"),
 arrows=2, free=TRUE, values = 0,
 labels=c("th_MH","th_MH","th_MH","th_MH","th_MH","th_MH","th_MH"))

) # Close Model   

blgm.math.hyp.fit <- mxRun(blgm.math.hyp.omx)

summary(blgm.math.hyp.fit)


#free parameters:
#      name matrix   row   col     Estimate   Std.Error lbound ubound
#1     th_M      S math2 math2 36.126276844 1.839264080              
#2    th_MH      S math2  hyp2 -0.010736501 0.233577456              
#3     th_H      S  hyp2  hyp2  1.104125879 0.056896723              
#4   psi_11      S eta_1 eta_1 64.711077297 5.414988264              
#5   psi_21      S eta_1 eta_2 -0.203518436 1.065187297              
#6   psi_22      S eta_2 eta_2  0.751586255 0.317819734              
#7   psi_31      S eta_1 eta_3 -2.979502771 0.677650648              
#8   psi_32      S eta_2 eta_3  0.097702665 0.160619040              
#9   psi_33      S eta_3 eta_3  1.542132569 0.155562786              
#10  psi_41      S eta_1 eta_4  0.107216732 0.164822192              
#11  psi_42      S eta_2 eta_4 -0.040159313 0.037905973              
#12  psi_43      S eta_3 eta_4 -0.019894808 0.030878829              
#13  psi_44      S eta_4 eta_4  0.004628299 0.008589374              
#14 alpha_1      M     1 eta_1 35.258686586 0.355496873              
#15 alpha_2      M     1 eta_2  4.342993508 0.088357340              
#16 alpha_3      M     1 eta_3  1.903468154 0.058198520              
#17 alpha_4      M     1 eta_4 -0.056592836 0.014327333              
#
#observed statistics:  4391 
#estimated parameters:  17 
#degrees of freedom:  4374 
#-2 log likelihood:  23580.95 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  14832.951           23614.95                   NA
#BIC:  -6330.233           23697.20             23643.21
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 


################# Linear Growth Model with Time-Varying Covariate ################# 
tvc.math.spr.omx <- mxModel("Time Varying Covariate Model, Path Specification",
type='RAM',
mxData(observed=nlsy_math_wide, type="raw"),
manifestVars=c('math2','math3','math4','math5','math6','math7','math8',
               'spring2','spring3','spring4','spring5','spring6','spring7','spring8'),
latentVars=c('eta_1','eta_2'),

# Residual variances of mathematics scores
mxPath(from=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=2, free=TRUE,
 values=60, labels='theta'),

# Latent variable variances and covariances 
mxPath(from=c('eta_1','eta_2'),
 arrows=2, connect='unique.pairs', free=TRUE, 
 values = c(80, 0, 1),
 labels=c('psi_11','psi_21','psi_22')),

# Factor Loadings 
mxPath(from='eta_1',
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2',
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# Latent Variable Means
mxPath(from='one',
 to=c('eta_1','eta_2'),
 arrows=1, free=TRUE, values=c(40, 5), labels=c('alpha_1','alpha_2')),

### TVC 
# means
mxPath(from='one',
 to=c('spring2','spring3','spring4','spring5','spring6','spring7','spring8'),
 arrows=1, free=TRUE,
 values=.5, labels=c('tau_1','tau_2','tau_3','tau_4','tau_5','tau_6','tau_7')),

# variances  
mxPath(from=c('spring2','spring3','spring4','spring5','spring6','spring7','spring8'),
 arrows=2, free=TRUE,
 values=.25, labels=c('phi_11','phi_22','phi_33','phi_44','phi_55','phi_66','phi_77')),

# Effect of TVC
mxPath(from=c('spring2','spring3','spring4','spring5','spring6','spring7','spring8'),
 to=c('math2','math3','math4','math5','math6','math7','math8'),
 arrows=1, free=TRUE, values=0, labels='beta')

) # Close Model

tvc.math.spr.fit <- mxRun(tvc.math.spr.omx)

summary(tvc.math.spr.fit)

#free parameters:
#      name matrix     row     col   Estimate  Std.Error lbound ubound
#1     beta      A   math2 spring2  4.6940553 0.39551314              
#2    theta      S   math2   math2 34.7210432 1.76320184              
#3   phi_11      S spring2 spring2  0.2381285 0.01839918              
#4   phi_22      S spring3 spring3  0.2479525 0.01689031              
#5   phi_33      S spring4 spring4  0.2288295 0.01664488              
#6   phi_44      S spring5 spring5  0.2239854 0.01642314              
#7   phi_55      S spring6 spring6  0.2068638 0.01481381              
#8   phi_66      S spring7 spring7  0.1681300 0.01807710              
#9   phi_77      S spring8 spring8  0.2023405 0.02401259              
#10  psi_11      S   eta_1   eta_1 55.7572326 4.93419959              
#11  psi_21      S   eta_1   eta_2  0.4679625 0.98984847              
#12  psi_22      S   eta_2   eta_2  0.6892008 0.30301173              
#13   tau_1      M       1 spring2  0.6089562 0.02666145              
#14   tau_2      M       1 spring3  0.5452433 0.02398478              
#15   tau_3      M       1 spring4  0.6455024 0.02460370              
#16   tau_4      M       1 spring5  0.6612901 0.02453773              
#17   tau_5      M       1 spring6  0.7076926 0.02303071              
#18   tau_6      M       1 spring7  0.7861269 0.03117384              
#19   tau_7      M       1 spring8  0.7183101 0.03774628              
#20 alpha_1      M       1   eta_1 32.6478655 0.40354415              
#21 alpha_2      M       1   eta_2  4.1586537 0.08758873              
#
#observed statistics:  4442 
#estimated parameters:  21 
#degrees of freedom:  4421 
#-2 log likelihood:  18757.58 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:    9915.58           18799.58                   NA
#BIC:  -11475.01           18901.19             18834.49
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-25 14:08:30 
#frontend time: 0.05300593 secs 
#backend time: 1.847184 secs 
#independent submodels time: 0 secs 
#wall clock time: 1.90019 secs 
#cpu time: 1.90019 secs 
#OpenMx version number: 999.0.0-3473 


















