nlsy_math_long <- read.table("C:/GRE2016/Data/nlsy_math_hyp_long_R.dat", na.string='.')
# Give the variable names
names(nlsy_math_long)<-c('id', 'female', 'lb_wght', 'anti_k1', 'math', 'comp', 'rec', 'bpi', 'as', 'anx', 'hd', 'hyp', 'dp', 'wd', 
                         'grade', 'occ', 'age', 'men', 'spring', 'anti')

summary(nlsy_math_long)

library(nlme)

# creating a multivariate form of the data
math <- data.frame(id     = nlsy_math_long$id, 
                   var    = nlsy_math_long$math, 
                   grade  = nlsy_math_long$grade,
                   d_math = 1, 
                   d_hyp  = 0, 
                   grp    = 'math')

hyp <- data.frame(id     = nlsy_math_long$id,
                  var    = nlsy_math_long$hyp,
                  grade  = nlsy_math_long$grade,
                  d_math = 0,
                  d_hyp  = 1,
                  grp    = 'hyp')

multivariate <- rbind(math, hyp)

summary(multivariate)
attach(multivariate)

# bivariate growth model
math.hyp.nlme<-nlme(var~d_math*(b_1i+b_2i*(grade-2))+
                        d_hyp *(h_1i+h_2i*(grade-2)),
                    data      = multivariate,
                    fixed     = b_1i+b_2i+h_1i+h_2i~1,
                    random    = b_1i+b_2i+h_1i+h_2i~1,
                    group     = ~id,
                    start     = c(35, 4, 1, -1),
                    weights   = varIdent(c(hyp=.3), form = ~1|grp),
                    na.action = na.omit)

summary(math.hyp.nlme) 

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: var ~ d_math * (b_1i + b_2i * (grade - 2)) + d_hyp * (h_1i +      h_2i * (grade - 2)) 
# Data: multivariate 
#       AIC      BIC    logLik
#  23612.95 23715.15 -11790.48
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, h_1i ~ 1, h_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev     Corr                
#b_1i     8.04436620 b_1i   b_2i   h_1i  
#b_2i     0.86704196 -0.029              
#h_1i     1.24184756 -0.299  0.093       
#h_2i     0.06810481  0.201 -0.698 -0.235
#Residual 6.01038401                     
#
#Variance function:
# Structure: Different standard deviations per stratum
# Formula: ~1 | grp 
# Parameter estimates:
#     math       hyp 
#1.0000000 0.1748218 
#Fixed effects: b_1i + b_2i + h_1i + h_2i ~ 1 
#        Value Std.Error   DF  t-value p-value
#b_1i 35.25849 0.3551053 3456 99.29024   0e+00
#b_2i  4.34307 0.0873764 3456 49.70531   0e+00
#h_1i  1.90349 0.0581508 3456 32.73367   0e+00
#h_2i -0.05663 0.0142229 3456 -3.98155   1e-04
# Correlation: 
#     b_1i   b_2i   h_1i  
#b_2i -0.531              
#h_1i -0.166  0.037       
#h_2i  0.040 -0.067 -0.608
#
#Standardized Within-Group Residuals:
#       Min         Q1        Med         Q3        Max 
#-3.3288003 -0.5329710 -0.0638618  0.5539257  3.0489620 
#
#Number of Observations: 4391
#Number of Groups: 932 


## Alternative Approach
math.hyp.nlme <- nlme(var~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2),
                      data=multivariate,
                      fixed=beta_1+beta_2 ~ grp-1,
                      random=d_1i+d_2i ~ grp-1,
                      groups = ~id,
                      start=c(35, 1, 4, -1),
                      weights = varIdent(c(hyp=.3),form =~ 1|grp),
                      na.action=na.omit)

summary(math.hyp.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: var ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) 
# Data: multivariate 
#       AIC      BIC    logLik
#  23612.95 23715.15 -11790.48
#
#Random effects:
# Formula: list(d_1i ~ grp - 1, d_2i ~ grp - 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#             StdDev     Corr                      
#d_1i.grpmath 8.04450689 d_1i.grpm d_1i.grph d_2i.g
#d_1i.grphyp  1.24205233 -0.299                    
#d_2i.grpmath 0.86716086 -0.029     0.093          
#d_2i.grphyp  0.06857084  0.199    -0.236    -0.691
#Residual     6.01030524                           
#
#Variance function:
# Structure: Different standard deviations per stratum
# Formula: ~1 | grp 
# Parameter estimates:
#     math       hyp 
#1.0000000 0.1748033 
#Fixed effects: beta_1 + beta_2 ~ grp - 1 
#                  Value Std.Error   DF  t-value p-value
#beta_1.grpmath 35.25848 0.3551077 3456 99.28952   0e+00
#beta_1.grphyp   1.90348 0.0581542 3456 32.73166   0e+00
#beta_2.grpmath  4.34308 0.0873776 3456 49.70477   0e+00
#beta_2.grphyp  -0.05662 0.0142254 3456 -3.98050   1e-04
# Correlation: 
#               bt_1.grpm bt_1.grph bt_2.grpm
#beta_1.grphyp  -0.166                       
#beta_2.grpmath -0.531     0.037             
#beta_2.grphyp   0.040    -0.608    -0.067   
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-3.32662576 -0.53305734 -0.06380582  0.55392230  3.04920021 
#
#Number of Observations: 4391
#Number of Groups: 932 



### Time-Varying Covariate Model
math.tvc.nlme<-nlme(math~(beta_1+d_1i)+(beta_2+d_2i)*(grade-2)+b3*spring,
                      data      = nlsy_math_long,
                      fixed     = beta_1+beta_2+b3~1,
                      random    = d_1i+d_2i~1,
                      group     = ~id,
                      start     = c(35, 4, 0),
                      na.action = na.omit)

summary(math.tvc.nlme)


#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_1 + d_1i) + (beta_2 + d_2i) * (grade - 2) + b3 *      spring 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15814.31 15854.25 -7900.155
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr 
#d_1i     7.4670761 d_1i 
#d_2ii     0.8301936 0.075
#Residual 5.8924537      
#
#Fixed effects: beta_1 + beta_2 + b3 ~ 1 
#          Value Std.Error   DF  t-value p-value
#beta_1 32.64789 0.4021616 1287 81.18102       0
#beta_2  4.15866 0.0868214 1287 47.89896       0
#b3      4.69404 0.3931911 1287 11.93830       0
# Correlation: 
#       beta_1 beta_2
#beta_2 -0.334       
#b3     -0.543 -0.179
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.265693646 -0.522918388 -0.003805591  0.534813023  2.679014672 
#
#Number of Observations: 2221
#Number of Groups: 932 
