ODS GRAPHICS OFF;
ODS HTML CLOSE;
ODS LISTING;

*Reading Data into SAS;
DATA nlsy_math_long;
	INFILE 'C:\GRE2016\Data\nlsy_math_hyp_long_R.dat';
	INPUT 
		id	female 		lb_wght		anti_k1

		math 	comp		rec		
		bpi		as 			anx 		hd 		hyp 		dp 		wd
		grade		occ			age
		men		spring		anti;
RUN;

* Fitting a linear growth model to mathematics scores;
PROC NLMIXED DATA = nlsy_math_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (grade - 2);

	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
									  c_21, v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 5
	
		v_1 = 70	v_2 = 1	c_21 = 0

		v_u = 40;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                  15937
                            AIC (smaller is better)            15949
                            AICC (smaller is better)           15949
                            BIC (smaller is better)            15978


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      35.2673    0.3554   930    99.23    <.0001    0.05   34.5698   35.9648   -0.0002
  beta_2       4.3393   0.08831   930    49.14    <.0001    0.05    4.1660    4.5126  -0.00036
  v_1         64.5617    5.6594   930    11.41    <.0001    0.05   53.4550   75.6683  5.631E-6
  v_2          0.7325    0.3273   930     2.24    0.0254    0.05   0.09024    1.3749  0.000022
  c_21        -0.1814    1.1501   930    -0.16    0.8747    0.05   -2.4384    2.0756  0.000081
  v_u         36.2298    1.8665   930    19.41    <.0001    0.05   32.5666   39.8929  -0.00001
*/

*Hyperactivity;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = nlsy_math_long;
	TITLE1 'PIAT Mathematics by Grade';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=50000
		Value=dot
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'BPI Hyperactivity')
		ORDER = (0 to 5 BY 1)
		MINOR = none
		OFFSET= (2 pct, 2 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Grade')
		ORDER = (2 to 8 BY 1)
		OFFSET= (5 PCT, 5 PCT)
		MINOR = none
		OFFSET= (2 pct, 2 pct);
	PLOT hyp * grade = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;


* Fitting a linear growth model to Hyperactivity;
PROC NLMIXED DATA = nlsy_math_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (grade - 2);

	MODEL hyp ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
									  c_21, v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 2	beta_2 = -.05
	
		v_1 = 1.5	v_2 = .05	c_21 = 0

		v_u = 1;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                 7688.7
                            AIC (smaller is better)           7700.7
                            AICC (smaller is better)          7700.8
                            BIC (smaller is better)           7729.7


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1       1.9018   0.05820   928    32.68    <.0001    0.05    1.7876    2.0160  -0.00045
  beta_2     -0.05540   0.01424   928    -3.89    0.0001    0.05  -0.08335  -0.02746  -0.00231
  v_1          1.5388    0.1555   928     9.89    <.0001    0.05    1.2335    1.8440  0.000337
  v_2        0.003766  0.008484   928     0.44    0.6572    0.05  -0.01288   0.02042  0.001414
  c_21       -0.01875   0.03076   928    -0.61    0.5423    0.05  -0.07912   0.04162  0.002671
  v_u          1.1086   0.05699   928    19.45    <.0001    0.05    0.9968    1.2205  -0.00017
*/

* Stacking data to fit a multivariate growth curve model;
DATA multivariate;
	SET nlsy_math_long;
	var = math;		d_math = 1;		d_hyp = 0; 		var_num = 1;	OUTPUT;
	var = hyp;		d_math = 0;		d_hyp = 1; 		var_num = 2;	OUTPUT;
	KEEP id grade age var d_math d_hyp var_num;
RUN;

* Multivariate Growth Curve Model;
PROC NLMIXED DATA = multivariate;
	traject = d_math * ((beta_1 + d_1i)  + (beta_2 + d_2i)  * (grade - 2)) +
              d_hyp  * ((alpha_1 + f_1i) + (alpha_2 + f_2i) * (grade - 2));

	IF d_math = 1 THEN v_e = v_u;
	IF d_math = 0 THEN v_e = v_s;

	MODEL var ~ NORMAL(traject, v_e);

	RANDOM d_1i d_2i f_1i f_2i
                   ~ NORMAL([0,0,0,0], [  v_d1,
                                        c_d2d1, v_d2,
                                        c_f1d1, c_f1d2, v_f1,
                                        c_f2d1, c_f2d2, c_f2f1, v_f2])
	SUBJECT = id;

	PARMS
       beta_1 = 35   beta_2 = 4      v_d1 = 65   v_d2 = .76  c_d2d1 = 0  v_u = 36.2
       alpha_1 = 2   alpha_2 = -.06  v_f1 = 1.6  v_f2 = .005  c_f2f1 = 0  v_s = 1.1
/* Bivariate Information */
       c_f1d1 = -3   c_f2d1 = 0      c_f1d2 = 0  c_f2d2 = 0;
RUN; 

/*
                                         Fit Statistics

                            -2 Log Likelihood                  23581
                            AIC (smaller is better)            23613
                            AICC (smaller is better)           23613
                            BIC (smaller is better)            23690


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      35.2457    0.3558   928    99.05    <.0001    0.05   34.5474   35.9441  -0.09656
  beta_2       4.3435   0.08792   928    49.40    <.0001    0.05    4.1709    4.5160   0.14042
  v_d1        64.9902    5.6863   928    11.43    <.0001    0.05   53.8307   76.1497  0.041092
  v_d2         0.6943    0.3221   928     2.16    0.0314    0.05   0.06205    1.3265  -0.57753
  c_d2d1     -0.06631    1.1417   928    -0.06    0.9537    0.05   -2.3069    2.1743  0.143932
  v_u         36.1982    1.8626   928    19.43    <.0001    0.05   32.5429   39.8536  -0.03121
  alpha_1      1.9043   0.05825   928    32.69    <.0001    0.05    1.7900    2.0187  0.053651
  alpha_2    -0.05681   0.01431   928    -3.97    <.0001    0.05  -0.08490  -0.02872  -0.44446
  v_f1         1.5471    0.1560   928     9.92    <.0001    0.05    1.2408    1.8533  0.216526
  v_f2       0.004728  0.008598   928     0.55    0.5825    0.05  -0.01215   0.02160  -0.79665
  c_f2f1     -0.02066   0.03096   928    -0.67    0.5047    0.05  -0.08142   0.04009  -0.23259
  v_s          1.1036   0.05685   928    19.41    <.0001    0.05    0.9920    1.2151   0.04637
  c_f1d1      -2.9902    0.6225   928    -4.80    <.0001    0.05   -4.2118   -1.7686  0.083706
  c_f2d1       0.1103    0.1511   928     0.73    0.4656    0.05   -0.1862    0.4067   0.24928
  c_f1d2      0.09491    0.1469   928     0.65    0.5185    0.05   -0.1935    0.3833  -0.16386
  c_f2d2     -0.04047   0.03008   928    -1.35    0.1788    0.05  -0.09951   0.01857   0.10773
*/


DATA multivariate1;
	SET multivariate;
	grade_c2 = grade - 2;
RUN;

PROC MIXED DATA = multivariate1 NOCLPRINT COVTEST ;
	CLASS id var_num grade;
	MODEL var = d_math grade_c2*d_math d_hyp d_hyp*grade_c2/SOLUTION NOINT DDFM = satterthwaite;
	RANDOM d_math grade_c2*d_math d_hyp d_hyp*grade_c2/SUBJECT = id TYPE = UN;
	REPEATED var_num / TYPE = UN SUBJECT = id*grade;
RUN;

/*
                                Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm    Subject     Estimate       Error     Value        Pr Z

               UN(1,1)     id           64.8393      5.6741     11.43      <.0001
               UN(2,1)     id           -0.2210      1.1562     -0.19      0.8484
               UN(2,2)     id            0.7591      0.3301      2.30      0.0107
               UN(3,1)     id           -2.9824      0.6743     -4.42      <.0001
               UN(3,2)     id           0.09774      0.1610      0.61      0.5437
               UN(3,3)     id            1.5456      0.1559      9.91      <.0001
               UN(4,1)     id            0.1071      0.1642      0.65      0.5144
               UN(4,2)     id          -0.04018     0.03795     -1.06      0.2896
               UN(4,3)     id          -0.02042     0.03096     -0.66      0.5096
               UN(4,4)     id          0.004832    0.008613      0.56      0.2874
               UN(1,1)     id*grade     36.1274      1.8637     19.38      <.0001
               UN(2,1)     id*grade    -0.01083      0.2331     -0.05      0.9629
               UN(2,2)     id*grade      1.1041     0.05692     19.40      <.0001


                                         Fit Statistics

                              -2 Res Log Likelihood         23595.6
                              AIC (smaller is better)       23621.6
                              AICC (smaller is better)      23621.7
                              BIC (smaller is better)       23684.5


                                Null Model Likelihood Ratio Test

                                  DF    Chi-Square      Pr > ChiSq

                                  12       6488.15          <.0001


                                   Solution for Fixed Effects

                                            Standard
             Effect             Estimate       Error      DF    t Value    Pr > |t|

             d_math              35.2585      0.3552     861      99.27      <.0001
             d_math*grade_c2      4.3432     0.08741     603      49.69      <.0001
             d_hyp                1.9035     0.05816     817      32.73      <.0001
             grade_c2*d_hyp     -0.05661     0.01423     565      -3.98      <.0001
*/




* Fitting a linear growth model to mathematics scores with a time-varying covariate;
PROC MEANS;
	CLASS grade;
	VAR spring;
RUN;
PROC NLMIXED DATA = nlsy_math_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (grade - 2) + b3 * spring;

    MODEL math ~ NORMAL(traject, v_u);
    RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
                                     c_21, v_2])
	SUBJECT = id;
	PARMS
        beta_1 = 40 beta_2 = 5 b3 = 0
        v_1 = 70    v_2 = 1    c_21 = 0
        v_u = 40;
RUN; 


/*
                                         Fit Statistics

                            -2 Log Likelihood                  15800
                            AIC (smaller is better)            15814
                            AICC (smaller is better)           15814
                            BIC (smaller is better)            15848


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      32.6479    0.4036   930    80.89    <.0001    0.05   31.8558   33.4400  0.000052
  beta_2       4.1587   0.08762   930    47.46    <.0001    0.05    3.9867    4.3306  0.000079
  b3           4.6940    0.3957   930    11.86    <.0001    0.05    3.9175    5.4705  0.000053
  v_1         55.7575    5.1325   930    10.86    <.0001    0.05   45.6849   65.8301   0.00003
  v_2          0.6892    0.3195   930     2.16    0.0312    0.05   0.06225    1.3161  -0.00022
  c_21         0.4679    1.0745   930     0.44    0.6633    0.05   -1.6408    2.5767  0.000105
  v_u         34.7210    1.8007   930    19.28    <.0001    0.05   31.1872   38.2549  -0.00001
*/



DATA mean_traj;
	DO t = 2 to 8 by .5;
		IF t = 2.0 THEN tvc = 0;
		IF t = 2.5 THEN tvc = 1;
		IF t = 3.0 THEN tvc = 0;
		IF t = 3.5 THEN tvc = 1;
		IF t = 4.0 THEN tvc = 0;
		IF t = 4.5 THEN tvc = 1;
		IF t = 5.0 THEN tvc = 0;
		IF t = 5.5 THEN tvc = 1;
		IF t = 6.0 THEN tvc = 0;
		IF t = 6.5 THEN tvc = 1;
		IF t = 7.0 THEN tvc = 0;
		IF t = 7.5 THEN tvc = 1;
		IF t = 8.0 THEN tvc = 0;
		IF t = 8.5 THEN tvc = 1;
		pred = 32.6479 + 4.1587 * (t - 2) + 4.69 * tvc;
		OUTPUT;
	END;
RUN;

*Plotting Scores over time;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = mean_traj;
	TITLE1 'PIAT Mathematics by Grade';
	SYMBOL1
		Interpol=join
		Color=black
		Repeat=50000
		Value=dot
		Height=.5
		Width=5;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'PIAT Mathematics')
		ORDER = (0 to 90 BY 10)
		MINOR = none;
	AXIS2
		LABEL = (F=ARIAL H = 2 'Grade')
		ORDER = (2 to 8 BY 1)
		OFFSET= (5 PCT, 5 PCT)
		MINOR = none;

	PLOT pred * t /NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;

* Allowing the effect of the time-varying covariate to vary over time;
DATA nlsy_math_long1;
	SET nlsy_math_long;

	IF grade = 2 THEN d1 = 1; ELSE d1 = 0;
	IF grade = 3 THEN d2 = 1; ELSE d2 = 0;
	IF grade = 4 THEN d3 = 1; ELSE d3 = 0;
	IF grade = 5 THEN d4 = 1; ELSE d4 = 0;
	IF grade = 6 THEN d5 = 1; ELSE d5 = 0;
	IF grade = 7 THEN d6 = 1; ELSE d6 = 0;
	IF grade = 8 THEN d7 = 1; ELSE d7 = 0;
RUN;

PROC NLMIXED DATA = nlsy_math_long1;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (grade - 2) + b31 * d1 * spring
                                        + b32 * d2 * spring
                                        + b33 * d3 * spring
                                        + b34 * d4 * spring
                                        + b35 * d5 * spring
                                        + b36 * d6 * spring
                                        + b37 * d7 * spring;

	MODEL math ~ NORMAL(traject, v_u);
    RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
                                     c_21, v_2])
	SUBJECT = id;
	PARMS
        beta_1 = 40 beta_2 = 5 
        b31 = 0 b32 = 0 b33 = 0 b34 = 0 
        b35 = 0 b36 = 0 b37 = 0
        v_1 = 70    v_2 = 1    c_21 = 0
        v_u = 40;
RUN; 

/*
                                         Fit Statistics

                            -2 Log Likelihood                  15671
                            AIC (smaller is better)            15697
                            AICC (smaller is better)           15697
                            BIC (smaller is better)            15760


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      31.5179    0.4829   930    65.27    <.0001    0.05   30.5702   32.4655  0.000098
  beta_2       4.6666    0.1566   930    29.81    <.0001    0.05    4.3593    4.9739  -0.00021
  b31          2.9880    0.6882   930     4.34    <.0001    0.05    1.6373    4.3386  0.000072
  b32          6.7557    0.5952   930    11.35    <.0001    0.05    5.5876    7.9238  2.858E-6
  b33          7.0424    0.5553   930    12.68    <.0001    0.05    5.9526    8.1322  0.000018
  b34          5.0598    0.5699   930     8.88    <.0001    0.05    3.9414    6.1783  -0.00002
  b35          3.6456    0.6270   930     5.81    <.0001    0.05    2.4150    4.8762  -0.00002
  b36          1.2387    0.8389   930     1.48    0.1401    0.05   -0.4077    2.8852  0.000076
  b37         -0.8707    1.0225   930    -0.85    0.3947    0.05   -2.8775    1.1361  -0.00013
  v_1         57.4818    4.9510   930    11.61    <.0001    0.05   47.7654   67.1982  0.000022
  v_2          1.0321    0.3225   930     3.20    0.0014    0.05    0.3993    1.6650  0.000053
  c_21        -0.2535    1.0462   930    -0.24    0.8086    0.05   -2.3067    1.7998  -4.54E-6
  v_u         30.4289    1.6373   930    18.58    <.0001    0.05   27.2157   33.6422  0.000044
*/
